# Configure docker targets
# Copyright (C) 2021 David Sugar <tychosoft@gmail.com>.
#
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without
# modifications, as long as this notice is preserved.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

find_program(DOCKER_EXEC NAMES docker podman)
if(NOT DEFINED DOCKER_TAG)
    set(DOCKER_TAG "latest")
endif()

if(NOT DEFINED DOCKER_REGISTRY)
    set(DOCKER_REGISTRY "local")
endif()

if(NOT DEFINED DOCKER_PLATFORMS)
    set(DOCKER_PLATFORMS "linux/amd64,linux/arm64")
endif()

if(NOT DEFINED DOCKER_DEPLOY)
    set(DOCKER_DEPLOY "quay.io/gnutelephony/${PROJECT_ARCHIVE}-server")
endif()

add_custom_target(docker-dist
    WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
    DEPENDS dist
    USES_TERMINAL
    COMMAND "${DOCKER_EXEC}" pull alpine:latest
)

add_custom_target(docker-images
    WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
    DEPENDS docker-dist
    USES_TERMINAL
    COMMAND "${DOCKER_EXEC}" build --build-arg VERSION="${PROJECT_VERSION}" -t "${DOCKER_REGISTRY}/${PROJECT_ARCHIVE}-server:${DOCKER_TAG}" "${CMAKE_CURRENT_SOURCE_DIR}"
)

add_custom_target(docker-scan
    DEPENDS docker-images
    USES_TERMINAL
    COMMAND "${DOCKER_EXEC}" scan "${DOCKER_REGISTRY}/${PROJECT_ARCHIVE}-server:${DOCKER_TAG}"
)

add_custom_target(docker-shell
    DEPENDS docker-images
    USES_TERMINAL
    COMMAND "${DOCKER_EXEC}" run -it --entrypoint /bin/ash "${DOCKER_REGISTRY}/${PROJECT_ARCHIVE}-server:${DOCKER_TAG}"
)

add_custom_target(docker-inspect
    COMMAND "${DOCKER_EXEC}" buildx imagetools inspect "${DOCKER_DEPLOY}"
)

add_custom_target(publish-docker
    WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}"
    DEPENDS docker-dist
    USES_TERMINAL
    COMMAND "${DOCKER_EXEC}" buildx build --build-arg VERSION="${PROJECT_VERSION}" --platform "${DOCKER_PLATFORMS}" --push -t "${DOCKER_DEPLOY}:${DOCKER_TAG}" "${CMAKE_CURRENT_SOURCE_DIR}"
)
