/*
 * Copyright (C) 2020-2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef	CALLS_HPP_
#define CALLS_HPP_

#include "compiler.hpp"

#include <string>

class keyfile;

namespace server {
class registry;
class event_t;

using reason_t = struct reason {
    inline reason(int exit_code = 487, std::string exit_text = "") :
    code(exit_code), text(std::move(exit_text)) {}
    int code;
    std::string text;
};

auto next_callid() -> uint64_t;
void exec_calls(const keyfile& config);

// call manager dispatch actions
namespace exec {
void stop_calls();
void reload_calls(const keyfile& config);
void create_call(uint64_t id, const event_t& event, const registry& entry);
void invite_call(uint64_t id, const registry& entry, unsigned rings = 0);
void invite_delayed(uint64_t id, const registry& entry, unsigned rings = 0);
void start_call(uint64_t id, const event_t& event, const registry& entry);
void hangup_call(uint64_t id, const reason_t& reason);
void close_invite(const event_t& event);
void release_invite(const event_t& event);
void trying_invite(const event_t& event);
void answer_invite(const event_t& event);
void ringing_invite(const event_t& event);
void ack_invite(const event_t& event);
void failed_invite(const event_t& event);
void redo_invite(const event_t& event);
void hangup_user(const registry& entry, const reason_t& reason);
void call_info(const event_t& event);
}} // end namespace

/*!
 * Coventry call state processing.
 * \file calls.hpp
 */
#endif
