/*
 * Copyright (C) 2020-2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef	MAIN_HPP_
#define MAIN_HPP_

#include "compiler.hpp"

#include <atomic>

#include <unistd.h>

namespace server {
extern std::atomic<bool> running;
extern std::atomic<bool> exiting;
extern bool ipv6;
extern uid_t nobody;
} // end namespace

/*!
 * Common definitions used by the main server.
 * \file main.hpp
 */

/*!
 * Common namespace for the server.
 * \namespace server
 */

/*!
 * Namespace for component thread dispatched actions.
 * \namespace server::exec
 */

/*!
 * Network utility functions of various sorts.
 * \namespace server::net
 */

/*!
 * Process management functions.
 * \namespace server::proc
 */

/*!
 * Utility namespace for miscellaneous functions.
 * \namespace server::util
 */

/*!
 * \mainpage Coventry
 * Coventry can be a residential IP telephony gateway or small office voip
 * server to interconnect SIP endpoints with advanced voice, messaging, and
 * facility automation services.
 *
 * A number of resources exist for project development:
 * - [Gitea site] https://git.gnutelephony.org/coventry
 * - [Issues] https://git.gnutelephony.org/coventry/issues
 * - [Packages] https://pkg.gnutelephony.org/coventry
 * - [Project Boards] https://git.gnutelephony.org/coventry/projects
 * - [Pull Requests] https://git.gnutelephony.org/coventry/pulls
 * - [Report Bugs] https://git.gnutelephony.org/coventry/issues/new
 * - [Wiki] https://codeberg.org/gnutelephony/coventry/wiki/_pages
 *
 * \author David Sugar <tychosoft@gmail.com>
 */
#endif
