/*
 * Copyright (C) 2020-2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UTILS_HPP
#define UTILS_HPP

#include "compiler.hpp"

#include <string>

namespace server {
class event_t;
}

namespace server::util {
inline auto get8(const uint8_t *ptr) {
    return *ptr;
}

inline void put8(uint8_t *ptr, uint8_t value) {
    *ptr = value;
}

auto get16(const uint8_t *ptr) -> uint16_t;
auto get32(const uint8_t *ptr) -> uint32_t;
auto get64(const uint8_t *ptr) -> uint64_t;
void set16(uint8_t *ptr, uint16_t value);
void set32(uint8_t *ptr, uint32_t value);
void set64(uint8_t *ptr, uint64_t value);

auto to_hex(const uint8_t *data, size_t size) -> std::string;
auto from_hex(std::string_view from, uint8_t *to, size_t size) -> size_t;
auto get_env(const char *id, size_t max = FILENAME_MAX) -> const char *;
auto idverify(const char *cp, size_t max) -> size_t;
auto u8verify(const char *cp, size_t max = 255) -> size_t;
auto u8verify(std::string_view view) -> size_t;
auto get_value(std::string_view text) -> int;
auto get_count(std::string_view count) -> uint32_t;
auto get_port(const char *cp, uint16_t port = 0) -> uint16_t;
auto get_period(const char *cp, unsigned period = 0) -> unsigned;
auto get_expires(std::string_view cp) -> time_t;
auto count(const char *cp, size_t max = 256) -> size_t;
auto sha256_secret(std::string_view userid, std::string_view realm, std::string_view secret) -> std::string;
auto sha256_digest(std::string_view secret, const event_t& event) -> std::string;
auto md5_secret(std::string_view userid, std::string_view realm, std::string_view secret) -> std::string;
auto md5_digest(std::string_view secret, const event_t& event) -> std::string;
auto is_file(const std::string& path) -> bool;
auto is_bool(std::string value) -> bool;
auto is_true(std::string_view value) -> bool;
auto set_string(char *cp, size_t max, std::string_view view) -> size_t;
auto set_string(char *cp, size_t max, const char *from) -> size_t;
auto get_priority() -> int;
auto set_priority(int priority) -> bool;
} // end namespace

/*!
 * Utility functions and support for server.
 * \file utils.hpp
 */
#endif
