/*
 * Copyright (C) 2020-2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "compiler.hpp"
#include "ipc.hpp"

#include <print.hpp>
#include <args.hpp>

#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>

namespace {
args::option helpflag('h',  "--help",       "display this list");
args::option althelp('?',   nullptr, nullptr);
} // end namespace

auto main(int argc, const char **argv) -> int   // NOLINT
{
    auto count = args::parse(argc, argv);
    if(is(helpflag) || is(althelp) || count) {
        args::help({"pbx-realm"}, {
            "pbx-realm " PROJECT_VERSION,
            "David Sugar <tychosoft@gmail.com",
            "Realm of pbx services"
        });
        return 0;
    }

    auto rshm = shm_open(SERVER_IPCPATH ".registry", O_RDONLY, 0660);
    if(rshm < 0)
        die(4, "failed");
    struct stat rino{};
    if(fstat(rshm, &rino) < 0)
        die(4, "offline");
    auto rmax = (rino.st_size - sizeof(ipc_sys)) / sizeof(ipc_reg);
    auto rmem = mmap(nullptr, sizeof(ipc_reg) * rmax + sizeof(ipc_sys), PROT_READ, MAP_SHARED, rshm, 0);
    auto reg = static_cast<ipc_reg*>(rmem);
    auto sys = reinterpret_cast<ipc_sys *>(&reg[rmax]);
    print("{}\n", sys->realm);
}
