/*
 * Copyright (C) 2017-2021 David Sugar <tychosoft@gmail.com>.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PRINT_HPP_
#define PRINT_HPP_

#include <iostream>
#include <string_view>

#if __has_include(<format>)
#include <format>
template<class... Args>
inline auto print_format(std::string_view fmt, const Args&... args) {
    return std::format(fmt, args...);
}
#else
#include <fmt/format.h>
template<class... Args>
auto print_format(std::string_view fmt, const Args&... args) {
    return fmt::format(fmt, args...);
}
#endif

template<class... Args>
auto print(std::string_view fmt, const Args&... args) -> std::ostream& {
    return std::cout << print_format(fmt, args...);
}

template<class... Args>
auto print(std::ostream& out, std::string_view fmt, const Args&... args) -> std::ostream& {
    return out << print_format(fmt, args...);
}

template<class... Args>
void die(int code, std::string_view fmt, const Args&... args) {
    std::cerr << print_format(fmt, args...);
    ::exit(code);
}

template<class Out, class... Args>
auto print_to(Out out, std::string_view fmt, const Args&... args) -> std::ostream& {
    return out << print_format(fmt, args...);
}

/*!
 * Generic print and control manipulators.  This makes it easy to write
 * output and fifo control messages using C++ stream operators.
 * \file print.hpp
 */
#endif
